<?php

namespace App\Controller;

use src\Controller\Controller;
use src\MyClass\Session;
use src\MyClass\Tools;

class InstallerController extends Controller
{
    private $current_menu;

    /**
     * @var Session
     */
    private $session;

    /**
     * @var mixed
     */
    private $viewJson;

    public function __construct(){
        $this->session = Session::getSessionInstance();
        // Lecture du fichier de conf Json (base de données)
        parent::__construct();
        $this->current_menu = 'home';
    }

    /**
     * Page d'accueil de l'installateur
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function home(){
        $checkExtensionIntl = extension_loaded('php7.x-intl');
        $checkExtensionXml = extension_loaded('php7.x-xml');
        $clientPHPVersion = PHP_VERSION;
        $checkPHP = version_compare(PHP_VERSION, '7.4.0', '>=');

        $this->render('home.twig', [
            'checkExtensionIntl' => $checkExtensionIntl,
            'checkExtensionXml' => $checkExtensionXml,
            'clientPHPVersion' => $clientPHPVersion,
            'checkPHP' => $checkPHP,
            'current_menu' => 'home'
        ]);
    }

    /**
     * Page d'accueil de l'installateur (licence)
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function licence(){
        $licence = htmlspecialchars(file_get_contents(ROOT_INSTALL . '/gpl.txt'));
        $this->render('licence.twig', [
            'licence' => $licence,
            'current_menu' => 'home'
        ]);
    }

    /**
     * Page d'accueil de l'installateur
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function unzip(){
        // Vérifie la version de PHP
        $this->checkPHP();

        if(isset($_POST['unzip'])){
            // Vérifie si le fichier .tar existe bien
            $checkTar = ROOT_INSTALL . '/proaxive_lite-beta.tar';
            if(file_exists($checkTar)){
                $this->session->setFlash('danger', "L'archive de Proaxive n'existe pas !");
                return $this->home();
            }
            // Récupération du fichier .tar
            $phar = new \PharData('../proaxive_lite-beta.tar');
            // Décompression du fichier au même niveau
            $phar->extractTo('../', null, true);
            if(!$phar){
                $this->session->setFlash('danger', 'Un problème inattendu est survenu !');
            }
            // Notification que c'est OK!
            $this->session->setFlash('success', 'Tous les fichiers ont été décompressés, <strong>veuillez patienter...</strong>');
            header("Location: /install/database");
        }

        $this->render('unzip.twig', [
            'current_menu' => 'unzip'
        ]);
    }

    /**
     * Inscrit les informations de la base de données dans le Json
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function database(){
        // Vérifie la version de PHP
        $this->checkPHP();
        // Vérifie si le dossier "vendor" existe bien
        $checkVendor = ROOT_APP . '/vendor';
        if(!file_exists($checkVendor)){
            $this->session->setFlash('danger', "Proaxive n'a pas été décompressé, ne brulez pas les étapes.");
            return $this->home();
        }
        // Récupération des données du formulaire
        if(!empty($_POST)){
            if(!empty($_POST['db_host']) && !empty($_POST['db_name']) && !empty($_POST['db_user']) && !empty($_POST['db_passwd'])){
                $writeJson = array(
                    'db_host' => htmlspecialchars($_POST['db_host'], ENT_QUOTES, 'UTF-8'),
                    'db_name' => htmlspecialchars($_POST['db_name'], ENT_QUOTES, 'UTF-8'),
                    'db_user' => htmlspecialchars($_POST['db_user'], ENT_QUOTES, 'UTF-8'),
                    'db_passwd' => $_POST['db_passwd']
                );
                // Vérifie si la connexion à la base de données est OK
                $postDB = $this->checkDB($_POST['db_name'], $_POST['db_host'], $_POST['db_user'], $_POST['db_passwd']);
                if($postDB != null){
                    // OK !
                    if($writeJson){
                        file_put_contents(ROOT_APP . 'src/Database/db.json', json_encode($writeJson));
                        $this->session->setFlash('success', 'Les informations ont bien été sauvegardées, <strong>veuillez patienter...</strong>');
                        header("Location: /install/importsql");
                    } else {
                        $this->session->setFlash('danger', 'Un souci, <strong>veuillez patienter...</strong>');
                        return $this->home();
                    }
                } else {
                    // Erreur -> retour PDO $e
                    $postDB;
                }
            } else {
                $this->session->setFlash('danger', "Le formulaire n'a pas été rempli complétement !");

            }
        }
        $this->render('database.twig', [
            'current_menu' => 'database'
        ]);
    }

    /**
     * Importe le fichiers .sql vers la base de données
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function importsql(){
        // Vérifie la version de PHP
        $this->checkPHP();

        // Vérifie si le fichier Json de configuration est rempli SI NON redirige vers
        // install/database
        if(!$this->readJson()->db_host OR !$this->readJson()->db_user){
            $this->session->setFlash('danger', "<strong>Erreur !</strong> Vérifiez vos informations de base de données et recommencez !");
            return $this->database();
        }
        // Importe les données du fichier .sql vers la base de données
        if (isset($_POST['importSql'])) {
            $db = new PDO('mysql:dbname=' . $this->readJson()->db_name . ';host='.$this->readJson()->db_host . '', $this->readJson()->db_user, $this->readJson()->db_passwd);
            $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $db->query("SET NAMES 'utf8', lc_time_names = 'fr_FR'");
            // Récupère le fichier .sql
            $query = file_get_contents(ROOT_INSTALL . '/config/proaxive_lite.mysql.sql');
            $stmt = $db->prepare($query);
            // Execute la tâche
            if ($stmt->execute()){
                $this->session->setFlash('success', '<strong>Terminé !</strong> Les tables ont été installées ! <strong>veuillez patienter...</strong>');
                $this->emptyJson();
                header("Location: /install/setting");
            }else{
                $this->session->setFlash('danger', "<strong>Erreur !</strong> Vérifiez vos informations de base de données et recommencez !");
            }
        }
        $this->render('importsql.twig', [
            'viewJson' => $this->readJson(),
            'current_menu' => 'importsql'
        ]);
    }

    /**
     * Permet de mettre à jour la base de données
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function updateDatabase(){
        // Vérifie la version de PHP
        $this->checkPHP();
        // Importe les données du fichier .sql vers la base de données
        if (isset($_POST['updateDatabase'])) {
            $db = new PDO('mysql:dbname=' . $this->readJson()->db_name . ';host='.$this->readJson()->db_host . '', $this->readJson()->db_user, $this->readJson()->db_passwd);
            $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $db->query("SET NAMES 'utf8', lc_time_names = 'fr_FR'");
            // Récupère le fichier .sql
            $query = file_get_contents(ROOT_INSTALL . '/config/proaxive_update.mysql.sql');
            $stmt = $db->prepare($query);
            // Execute la tâche
            if ($stmt->execute()){
                $this->session->setFlash('success', '<strong>Terminé !</strong> Les tables ont été installées ! <strong>veuillez patienter...</strong>');
                $this->emptyJson();
                header("Location: /install/setting");
            }else{
                $this->session->setFlash('danger', "<strong>Erreur !</strong> Vérifiez vos informations de base de données et recommencez !");
            }
        }

        $this->render('updatedatabase.twig', [
            'viewJson' => $this->viewJson,
            'current_menu' => 'updatedatabase'
        ]);
    }

    /**
     * Permet de choisir quelques parametres
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function setting(){
        // Vérifie si le fichier .tar existe bien
        $checkTar = ROOT_APP . 'src';
        // Vérifie la version de PHP
        $this->checkPHP();
        // Vérifie si le fichier Json de configuration est rempli SI NON redirige vers
        // install/database
        if(!$this->readJson()->db_host OR !$this->readJson()->db_user){
            $this->session->setFlash('danger', "<strong>Erreur !</strong> Vérifiez vos informations de base de données et recommencez !");
            return $this->database();
        }

        if(!empty($_POST)){
            $writeJson = array(
                'php_error' => $_POST['php_error'],
                'full_error' => $_POST['full_error'],
                'app_url' => htmlspecialchars($_POST['app_url'], ENT_QUOTES, 'UTF-8'),
                'style_selector' => 0,
                'view_version' => 1,
                'app_login_url' => 'login-dash'
            );
            if($writeJson){
                file_put_contents(ROOT_APP. '/config/setting.json', json_encode($writeJson));
                $this->session->setFlash('success', "Paramètres sauvegardés...<strong>Veuillez patienter...</strong> !");
                header("Location: /install/finish");
            } else {
                $this->session->setFlash('danger', "Un problème est survenu !");
            }

        }
        $this->render('setting.twig', [
            'current_menu' => 'setting'
        ]);
    }

    /**
     * Met fin à l'installation de l'application
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    public function finish(){
        // Vérifie si le fichier .tar existe bien
        $checkTar = ROOT_APP . 'src';
        // Vérifie la version de PHP
        $this->checkPHP();
        // Vérifie si le fichier Json de configuration est rempli SI NON redirige vers
        // install/database
        if(!$this->readJson()->db_host OR !$this->readJson()->db_user){
            $this->session->setFlash('danger', "<strong>Erreur !</strong> Vérifiez vos informations de base de données et recommencez !");
            return $this->database();
        }

        if(!empty($_POST)){
            rename(ROOT_APP . "/.htaccess.txt", ROOT_APP. "/.htaccess");
            rename(ROOT_APP . "/.env.exemple", ROOT_APP. "/.env");
            unlink(ROOT_APP . "/proaxive_lite-beta.tar");
            $this->session->setFlash('success', 'Installation terminée !...<strong>veuillez patienter...</strong>');
            header("Refresh: 2;url=/login-dash");
        }

        $this->render('finish.twig', [
            'current_menu' => 'finish'
        ]);
    }

    /**
     * $db_name, $db_host, $db_user, $db_passwd
     * @param $db_name
     * @param $db_host
     * @param $db_user
     * @param $db_passwd
     * @return PDO
     */
    private function checkDB($db_name, $db_host, $db_user, $db_passwd){
        try{
            $db = new PDO('mysql:dbname=' . $db_name . ';host='.$db_host . '', $db_user, $db_passwd);
            $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            return $db;
        } catch (\PDOException $e){
            $this->session->setFlash('danger', 'Impossible de se connecter à la base de données, <strong>veuillez vérifier vos informations de connexion</strong> : '. $e->getMessage());
        }
    }

    /**
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     */
    private function checkPHP(){
        $checkPHP = version_compare(PHP_VERSION, '7.4.0', '>=');
        if(!$checkPHP){
            return $this->home();
        }
    }

    /**
     *
     */
    private function readJson(){
        $viewJsonFile = file_get_contents(ROOT_APP . '/src/Database/db.json');
        return json_decode($viewJsonFile, false);
    }

    /**
     * Réinitialise le fichier de configuration Json
     */
    private function emptyJson(){
        $writeJson = array(
            'db_host' => '',
            'db_name' => '',
            'db_user' => '',
            'db_passwd' => ''
        );
        file_put_contents(ROOT_INSTALL . '/config/setting.json', json_encode($writeJson));
    }
}